'use strict';

const _ = require('lodash');
const Swagger = require('swagger-client');

const Service = require('../service');
const Cache = require('../cache');

class SwaggerApi extends Service {
  constructor(options, context) {
    super(options, context);

    this.swagger = new Swagger({
      url: options.url,
      spec: options.spec,
      usePromise: true
    });
  }

  call(methodName, parameters, options) {
    const api = _.get(this.options, 'api', 'default');
    const callOptions = {};

    _.defaults(parameters, this.options.defaultParameters);

    if (options.authorize) {
      callOptions.clientAuthorizations = _.mapValues(options.authorize, (value, name) => {
        const authorizer = _.get(this.options.authorizers, name);

        if (!authorizer) {
          throw new Error(`Authorizer ${name} not found`);
        }

        const field = _.get(authorizer, 'field', name);
        const type = _.get(authorizer, 'type');
        switch (type) {
          case 'header':
            return new Swagger.ApiKeyAuthorization(field, value, 'header');
          case 'query':
            return new Swagger.ApiKeyAuthorization(field, value, 'query');
          case 'basic':
            const username = _.get(value, 'username');
            const password = _.get(value, 'password');
            return new Swagger.PasswordAuthorization(username, password);
          case 'method':
            if (!_.isFunction(authorizer.method)) {
              throw new Error('No authorizer method provided');
            }

            return authorizer.method(value, parameters);
          default:
            throw new Error(`Unknown authorizer type ${type}`)
        }
      });
    }

    return this.swagger.then((client)=>{
      if (!_.has(client, [api, methodName])) {
        throw new Error('Api method not found');
      }

      return client[api][methodName](parameters, callOptions).then(result => {
        return {data: result.obj, info: result};
      });
    });
  }
}

module.exports = SwaggerApi;
